import Crypt;

/**	<p>Diese Klasse stellt eine Implementierung des mareilschen Verschlsselungsalgorithmus dar.
Der mareilsche Algorithmus erlaubt das Ver- und Entschlsseln von 64 Byte groen Datenblcken. 
Der Algorithmus wird in der entsprechenden <a href="vsc.html">Dokumentation</a> genauer erklrt.
Diese Klasse implementiert eine Standardschnittstelle (Crypt).</p>
<p>
Diese Implementation trgt den Namen VSC (Very Simple Crypt), man sollte sich aber auf keinen 
Fall vom Namen abschrecken lassen. Der Algorithmus ist tatschlich sehr einfach zu Implementieren, 
allerdings keineswegs Trivial. Es drfte relativ schwierig werden diesen Algorithmus zu brechen. 
Bis heute ist mir jedenfalls noch keine Methode zum Brechen des Algorithmus bekannt. Wer einen 
guten Ansatz gefunden hat kann mir eine Mail schreiben, an: 
<a href="mailto:Alexander.Weber@myokay.net">Alexander.Weber@myokay.net</a>.</p>
<p>
Dadurch das dieser Algorithmus in Java realisiert wurde ist er zwar etwas langsamer als andere
nicht in Java realisierte Algorithmen, er drfte aber schnell genug sein, auerdem ist er im 
Internet leicht anwendbar und ist sogar ohne Probleme in Java-Script zu realisieren. Der Vorteil 
dieses Algorithmus ist die Mglichkeit bei einer fortwhrenden Datenbertragung jeden Datenblock,
also alle 64 Byte Blcke mit unterschiedlichen Schlsseln zu verschlsseln, dadurch werden zwei
gleiche Quelldatenblcke nie gleich in der Verschlsselung aussehen, dies erschwert das
Brechen der Verschlsselung erheblich. Der Nachteil ist das das Kennwort von beiden Seiten auf 
jeden Fall geheim gehalten werden mu, da es sich nicht um einen Public-Key Algorithmus handelt.</p>
<p>
<hr>
    This program is free software; you can redistribute it and/or modify							<br>
    it under the terms of the GNU General Public License as published by							<br>
    the Free Software Foundation; either version 2 of the License, or								<br>
    (at your option) any later version.																<br>
																									<br>
    This program is distributed in the hope that it will be useful,									<br>
    but WITHOUT ANY WARRANTY; without even the implied warranty of									<br>
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the									<br>
    GNU General Public License for more details.													<br>
																									<br>
    You should have received a copy of the GNU General Public License								<br>
    along with this program; if not, write to the Free Software										<br>
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA						<br>
<hr>
</p>



<p>
<b>Bekannte Fehler:</b></p>
<p><table border=1>

<tr><th align=left>
keine Fehler bekannt.
</th></tr>

</table></p>



<p>
<b>nderungen an der Klasse:</b></p>
<p><table border=1>


<tr><th align=left width=20%>02.12.2000<br>Alexander Weber<br>--[Build 2]--</th>
<th align=left>
	Ich habe einen Fehler in der Methode <i>QueryCryptString</i> ausgebaut, dieser Fehler war ein
	Vergleich mit > 0, er mu lauten >= 0. Weiterhin wurde ein Fehler in der Methode 
	<i>isValidCryptKey</i> behoben, wo ich vergessen hatte einen Zeiger gegen null zu prfen.
	Weitere nderungen an der Methode <i>CompleteCryptKey</i>, auch dort wurde nicht auf einen 
	null-Zeiger geprft, auerdem wurde auf < 3 abgefragt, was aber gegen <= 3 sein sollte. Die
	Methode <i>isSafeCryptString</i> wird implementiert, wie in der neuen Version (Build 2) der 
	Crypt-Schnittstelle verlangt.
</th></tr>


<tr><th align=left>29.11.2000<br>Alexander Weber<br>--[Build 1]--</th>
<th align=left>
	Die Klasse wird von mir zum ersten mal released.
</th></tr>


</table></p>


@author (C) 2000, <a href="mailto:Alexander.Weber@myokay.net">Alexander Weber</a>
@version Version 1.0alpha (Build 2)
*/
public class BlockCrypt
implements Crypt
{

//-------------------------------------------------------------------------------------------------
	/** Diese Methode wandelt ein Zeichen in einen Zeichenschlssel um.
	Ein Zeichenschlssel ist eine 32-Bit Ganzzahl (int), aber nur 6-Bit lang. Dabei werden nur die 
	Zeichen 'A' bis 'Z' in die Zahlen 0 bis 25 umgewandelt, die Zeichen 'a' bis 'z' in die Zeichen 
	26 bis 51, die Zeichen (Ziffern) '0' bis '9' in die Zahlen 52 bis 61 und das Leerzeichen als 
	62 sowie der Punkt (Zeichen '.') in die Zahl 63.
	Wenn das bergebene Zeichen keines der oben genannten Zeichen ist, dann liefert die	Methode 
	einen ungltigen Schlssel mit dem Wert -1 zurck.
	@param cDigit		Das zu bergebende Zeichen, welches in einen Zeichenschlssel umgewandelt
						wird.
	@return 			Die Methode liefert den Zeichenschlssel zurck oder -1, falls ein 
						ungltiges Zeichen an die Methode bergeben wurde.
	*/
	public int QueryCryptChar( char cDigit )
	{
		int result = -1;

		if( (cDigit >= 'A') && (cDigit <= 'Z') ) result = (int)cDigit - (int)'A';
		if( (cDigit >= 'a') && (cDigit <= 'z') ) result = ((int)cDigit - (int)'a') + 26;
		if( (cDigit >= '0') && (cDigit <= '9') ) result = ((int)cDigit - (int)'0') + 52;
		if( cDigit == ' ' ) result = 62;
		if( cDigit == '.' ) result = 63;
		return result;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Methode wandelt eine Zeichenkette in eine Zeichenschlsselfolge um.
	Dabei werden alle einzelnen Zeichen der bergebenen Zeichenkette in die entsprechenden 
	Zeichenschlssel bersetzt, ungltige Zeichen, also Zeichen die nicht der Zeichenspezifikation
	(siehe QueryCryptChar) entsprechen, werden ignoriert. Es werden immer 768 Zeichenschlssel 
	erzeugt, da dies der bentigten Anzahl an Zeichenschlsseln fr einen Crypt-Key entspricht.
	@param sPassword	Hier wird die Zeichenkette bergeben, welche in einen Zeichenschlssel
						bersetzt werden soll, blicherweise handelt es sich dabei um ein
						vom Benutzer eingegebenes Passwort.
	@return				Die Methode liefert ein Array bestehend aus 32-Bit Zeichenschlsseln
						zurck das 768 Eintrge enthlt, dabei kann ein Teil der Schlssel ungltig
						sein, also den Wert -1 enthalten.
	@see BlockCrypt#QueryCryptChar
	*/
	public int[] QueryCryptString( String sPassword )
	{
		int[] Keys = new int[768];				// Array fr alle Schlssel
		int i, j;								// Zhler
		int realkeys = 0;						// Anzahl der gltigen Zeichenschlssel

		// Zunhst wird das komplette Array fr die Zeichenschlssel mit ungltigen
		// Zeichenschlsseln gefllt.
		//
		for( i=0; i < 768; i++ ) Keys[i] = -1;

		// Jetzt werden alle gltigen Zeichen in Zeichenschlssel bersetzen und in das dafr
		// reservierte Array geschrieben, ungltige Zeichen ignorieren.
		//
		for( i=0; i < sPassword.length(); i++ )
		{
			Keys[realkeys] = QueryCryptChar( sPassword.charAt(i) );
			if( Keys[i] >= 0 ) realkeys++;
		}

		return Keys;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Methode wandelt ein Array aus Zeichenschlsseln in einen Crypt-Key um.
	Zu diesem Zweck werden drei der jeweils 6-Bit langen Zeichenschlssel zu einem Teilschlssel
	zusammengefasst (siehe <a href="VSC-Dokumentation.html">VSC-Crypt Dokumentation</a>). Dadurch
	entstehen eine bestimmte Anzahl an Teilschlsseln die den Crypt-Key bilden. Der Crypt-Key
	besteht jedoch immer aus 256 18-Bit langen Teilschlsseln. Da jedoch nur Anzahl der 
	Zeichenschlssel / 3 Teilschlssel erzeugt werden knnen, werden die restlichen Teilschlssel
	durch ungltige Schlssel (-1) ersetzt. Ungltige Zeichenschlssel (alle Zeichenschlssel
	kleiner 0 und grer 63) im StringKey werden ignoriert.
	@param StringKey	Hier wird ein Array aus Zeichenschlsseln bergeben, blicherweise wurde
						dieses Array aus einem Benutzerkennwort ermittelt.
	@return				Die Methode liefert ein Array bestehend aus 32-Bit Teilschlsseln
						zurck das 256 Eintrge enthlt, dabei kann ein Teil der Schlssel ungltig
						sein, also den Wert -1 enthalten. Zusammen ergibt das Array den Crypt-Key. 
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	*/
	public int[] QueryCryptKey( int[] StringKey )
	{
		int[] CryptKey = new int[256];			// Array fr die Teilschlssel
		int i, j, pos;							// Zhlervariabeln
		int[] tmp_key = new int[3];				// Temporrvariabel fr die drei Zeichenschlssel
												// die zur Berechnung eines Teilschlssels ntig
												// sind

		// Zunchst fllen wir alle Teilschlssel mit ungltigen Schlsseln auf
		//
		for( i=0; i < 256; i++ ) CryptKey[i] = -1;

		// Wenn ein ungltiger Zeiger bergeben wurde das "ungltige" Crypt-Array zurckliefern.+
		//
		if( StringKey == null ) return CryptKey;

		// Jetzt berechnen wir jeden der 256 bentigen Teilschlssel, dazu verwenden wir die
		// Zeichenschlssel.
		//
		pos = 0;
		for( i=0; i < 256; i++ )
		{
			// Wir suchen die drei nchsten verwendbaren Zeichenschlssel.
			//
			for( j=0; j < 3; j++ )
			{
				// Den nchsten verwendbaren Zeichenschlssel finden.
				//
				while(	(pos < StringKey.length) && ((StringKey[pos] < 0) || (StringKey[pos] > 63)) ) pos++;
				
				// Wenn kein Teilschlssel mehr vorhanden, dann die Methode verlassen und den bisher
				// erzeugten Crypt-Key zurckliefern, sonst ist der gefundene Zeichenschlssel 
				// verwendbar.
				//
				if( pos >= StringKey.length ) return CryptKey;
				tmp_key[j] = StringKey[pos];
				pos++;
			}
			
			// Wenn wir drei verwendbare Zeichenschlssel gefunden haben, dann knnen wir
			// den nchsten Teilschlssel berechnen.
			//
			CryptKey[i] = (tmp_key[0] << 12) + (tmp_key[1] << 6) + tmp_key[2];
		}

		// Wenn wir alle 256-Schlssel erzeugen konnten, dann sind wir echt froh ! :-)
		//
		return CryptKey;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Version der QueryCryptKey-Methode erzeugt den Crypt-Key aus einer Zeichenkette.
	Sie verhlt sich exakt wie die andere Version der Methode, nur das Sie die bergebene
	Zeichenkette zuvor durch Aufruf der Methode QueryCryptString in einen Zeichenschlssel
	umgewandelt wird.
	@param StringKey	Hier wird ein Array aus Zeichenschlsseln bergeben, blicherweise wurde
						dieses Array aus einem Benutzerkennwort ermittelt.
	@return				Die Methode liefert ein Array bestehend aus 32-Bit Teilschlsseln
						zurck das 256 Eintrge enthlt, dabei kann ein Teil der Schlssel ungltig
						sein, also den Wert -1 enthalten. Zusammen ergibt das Array den Crypt-Key. 
	@see BlockCrypt#QueryCryptString
	@see BlockCrypt#QueryCryptKey( int[] )
	*/
	public int[] QueryCryptKey( String sPassword )
	{
		int CryptKey[];

		CryptKey = QueryCryptString( sPassword );
		return QueryCryptKey( CryptKey );
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Methode ergnzt fehlende Teilschlssel in einem Crypt-Key.
	Der Methode mu mindestens ein Crypt-Key mit 3 Teilschlsseln bergeben werden, wenn die
	Anzahl der bergebenen Teilschlssel weniger als 3 sind, dann wirft die Methode eine
	Ausnahme vom Typ ArrayIndexOutOfBoundsException. Sonst wird ein vollstndiger Crypt-Key aus
	dem vorhandenen Crypt-Key erzeugt. Die dazu verwendete Methode kann in der 
	<a href="VSC-Dokumentation.html">VSC-Crypt Dokumentation</a> nachgelesen werden.
	@param CryptKey		Hier wird ein Array aus 32-Bit Teilschlsseln bergeben, blicherweise 
						wurde dieses Array aus einem Benutzerkennwort ermittelt und von der 
						Methode QueryCryptKey zurckgeliefert.
	@return				Die Methode liefert ein Array bestehend aus 32-Bit Teilschlsseln
						zurck das 256 gltige Eintrge enthlt. Zusammen ergibt das Array den 
						gltigen Crypt-Key. 
	@exception ArrayIndexOutOfBoundsException 
						Diese Exception wird angestossen, wenn ein Crypt-Key mit weniger als
						drei Teilschlsseln bergeben wird.
	@see BlockCrypt#QueryCryptKey
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	*/
	public int[] CompleteCryptKey( int[] CryptKey )
	throws ArrayIndexOutOfBoundsException
	{
		int i;									// Zhler
		int keys = 0;							// Anzahl der vorhandenen Schlssel
		int[] fullkey = new int[256];			// Array welches den kompletten Schlssel aufnimmt

		// Wenn gar keine Teilschlssel bergeben wurde, dann Ausnahme werfen
		//
		if( CryptKey == null )
		{
			throw new ArrayIndexOutOfBoundsException("Mindestens 3 Teilschluessel werden erwartet");
		}

		// Jetzt werden wir die Anzahl der gltigen Teilschlssel im bergebenen Array
		// ermitteln.
		//
		for( i=0; i < CryptKey.length; i++ )
		{
			// Wenn der Teilschlssel ok ist, dann merken, sonst ignorieren
			//
			if( (CryptKey[i] >= 0) && (CryptKey[i] <= 0x3ffff) )
			{
				fullkey[keys] = CryptKey[i];
				keys++;
			}

			// Wenn bereits alle 256 Teilschlssel gefunden wurden, dann brauchen wir nicht 
			// mehr weiter zu suchen und brechen die Schleife ab, die Methode kann jetzt
			// auch schon den kompletten Schlssel zurckliefern.
			if( keys == 256 ) return fullkey;
		}

		// Wenn weniger als 3 Teilschlssel bergeben wurden, dann eine Ausnahme feuern und
		// die Methode damit verlassen.
		//
		if( keys <= 3 )
		{
			throw new ArrayIndexOutOfBoundsException("Mindestens 3 Teilschluessel werden erwartet");
		}

		// Jetzt men wir die restlichen Teilschlssel berechnen.
		//
		while( keys < 256 )
		{
			// Wir verdoppeln die vorhandenen Teilschlssel so lange bis die Anzahl der bentigen
			// Teilschlssel erzeugt wurde.
			//
			int n = keys;
			i = 0;
			while(  (i < (n/2)) && (keys < 256)  )
			{
				// Einfache Mathematik: Der neue Schlssel ist gleich zwei alter Schlssel
				// XOR-Verknpft miteinander, anschlieend werden die unwichtigen ungtligen
				// Bits via UND-Verknpfung wieder auf 0 zurckgesetzt. Dies wird gemacht weil
				// eigentlich nur die Bits 0 bis 17 bentigt werden und es ist einfach sauberer
				// wenn dann die unbenutzten Bits den Wert 0 enthalten. Des weiteren entstehen
				// keine ungtligen Schlssel.
				fullkey[ n+i ] = (fullkey[i] ^ fullkey[(n-1)-i]) & 0x3ffff;
				i++;
				keys++;
			}
		}
		return fullkey;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Methode liefert aus einer Zeichenkette einen vollstndigen Crypt-Key.
	Die Methode benutzt dazu einfach die Methoden QueryCryptKey und CompleteCryptKey. Die
	bergebene Zeichenkette sollte nur aus den in QueryCryptChar spezifizierten Zeichen bestehen
	und mindestens 12 gltige Zeichen enthalten. Wenn weniger als 12 gltige Zeichen in der
	Zeichenkette vorhanden sind kann kein gltiger Crypt-Key erzeugt werden, die Methode
	liefert dann null zurck.
	@param sPassword	Hier wird eine Zeichenkette bergeben, vornehmlich das von einem Benutzer 
						eingebene Kennwort, aus welcher dann ein Crypt-Key erzeugt wird. Aus ein
						un der selben Zeichenkette wird immer der gleiche Crypt-Key generiert.
	@return				Die Methode liefert ein Array bestehend aus 32-Bit Teilschlsseln
						zurck das 256 Eintrge enthlt und einen gltigen Crypt-Key darstellt,
						oder sie liefert null zurck, falls kein gltiger Crypt-Key generiert
						werden konnte.
	@see BlockCrypt#QueryCryptChar
	*/
	public int[] QueryCompleteCryptKey( String sPassword )
	{
		int CryptKey[] = null;

		try
		{
			CryptKey = QueryCryptKey( sPassword );
			CryptKey = CompleteCryptKey( CryptKey );
		}
		catch( ArrayIndexOutOfBoundsException e )
		{
			CryptKey = null;
		}
		return CryptKey;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Methode berprft einen Crypt-Key auf seine korrektheit.
	Ein Crypt-Key ist dann gltig, wenn er aus exakt 256 32-Bit groen Teilschlsseln besteht, 
	in denen nur die untersten 18-Bit benutzt sind. Genau dies berprft diese Methode, wenn
	der bergebene Crypt-Key weniger oder mehr als 256 Elemente besitzt ist er ungltig, die
	Methode liefert false zurck. Wenn einer der Teilschlssel auerhalb der spezifizierten
	Reichweiter (grer/gleich 0 und kleiner/gleich 0x3ffff) liegt, dann liefert die Methode
	ebenfalls false zurck. Nur wenn der Crypt-Key 100%tig korrekt ist, dann liefert die
	Methode true zurck. Sie kann dabei nicht entscheiden ob ein Schlssel sicher oder eher
	unsicher ist.
	@param CryptKey		Hier wird der zu berprfende CryptKey bergeben, ein Array aus 32-Bit 
						Teilschlsseln.
	@return				Die Methode liefert true zurck, wenn der bergebene Crypt-Key gltig
						ist, also den Spezifikationen des VSC-Algorithmus entspricht. In einem
						anderen Fall liefert die Methode false zurck.
	@see BlockCrypt#QueryCryptKey
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	*/
	public boolean isValidCryptKey( int[] CryptKey )
	{
		int i;									// Zhler

		// Wenn weniger oder mehr als 256 Schlssel vorhanden sind, dann ist der Crypt-Key
		// ungltig, also Methode beenden und false zurckgeben.
		//
		if( CryptKey == null ) return false;
		if( CryptKey.length != 256 ) return false;

		// Jetzt berprfen wir jeden einzelnen Crypt-Key auf seine korrektheit, sobalt einer
		// ungltig ist brechen wir die Methode ab und geben false zurck.
		//
		for( i=0; i < 256; i++ )
			if( (CryptKey[i] < 0) && (CryptKey[i] > 0x3ffff) ) return false;

		// Wenn alle Teilschlssel ok sind, dann zurck mit true.
		//
		return true;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	<p>Diese Methode verschlsselt einen 64-Byte groen Datenblock.
	Der Methode mu ein 64-Byte Datenblock und ein gltiger Crypt-Key bergeben werden. Der 
	Datenblock sollte aus 32 Worten (16-Bit Werten) bestehen. Die Methode kann nur zum 
	Verschlsseln verwendet werden. Die Methode liefert im Erfolgsfall den verschlsselten 
	Datenblock zurck, welcher gleich dem bergebenen Datenblock ist, sonst null. Null wird dann 
	zurckgeliefert, wenn die Methode einen ungltigen Datenblock (grer oder kleiner 32 Worten) 
	oder einen ungltigen Crypt-Key bergeben bekommt.</p>
	<p>
	<b>Wichtiger Hinweis:</b><br>
	Die Methode verndert den CryptKey dahingehend das dieser rotiert wird. Die Methode zum
	Entschlsseln (DecryptBlock) erwartet einen solchen rotierten CryptKey zum Entschlsseln. 
	Der CryptKey wird jediglich rotiert, wenn der von dieser Methode vernderte	CryptKey nicht 
	zur Verfgung steht (z.B. bei einer Onlineverbindung, wo nur der original CryptKey bekannt ist,
	weil nur er aus dem Kennwort errechnet werden kann), dann kann dieser Key durch den Aufruf der 
	Methode "RotateCryptKey" knstlich erzeugt werden. Um den original Crypt-Key wiederherzustellen 
	kann die Methode "DerotateCryptKey" verwendet werden, welche den Crypt-Key wieder in die 
	entgegengesetzte Richtung rotiert.</p>
	<p>
	Der Vorteil dieses Verhaltens ist eine etwas hhere Performance in einige Situationen, der
	einfachste Weg immer mit einem Schlssel zu arbeiten ist einen EncryptKey und einen DecryptKey
	zu erzeugen und je nach bedarf nur eine Kopie des Originals an die Methoden EncryptBlock und 
	DecryptBlock zu bergeben.</p>
	@param Data			Der Datenblock der verschlsselt werden soll, dieser Block besteht aus 32
						Worten (16-Bit Werte).
	@param CryptKey		Der CryptKey mit welchem verschlsselt werden soll, nach erfolgreichem
						Aufruf dieser Methode enthlt dieses Array einen rotierten Crypt-Key.
	@return				Die Methode liefert 32 Worte (16-Bit Werte) zurck, wenn die 
						Verschlsselung erfolgreich durchgefhrt wurde, dabei handelt es sich um
						den bergebenen Datenblock, sonst null.
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	@see BlockCrypt#DecryptBlock
	@see BlockCrypt#RotateCryptKey
	@see BlockCrypt#DerotateCryptKey
	*/
	public char[] EncryptBlock( char[] Data, int[] CryptKey )
	{
		int i, j;								// Zhler
		char xchg = 0;							// Variabel zum vertauschen von zwei Datenworten
		int key;								// Schlssel selbst
		int key_cmd;							// Befehl aus einem Schlssel
		int key_subround;						// Teilrundenzhler
		int key_round;							// Rundenzhler
		int[] key_cpy = new int[256];			// Array zum Kopieren des Crypt-Key

		// Wenn der bergebene Datenblock oder der Crypt-Key ungltig ist, dann die Methode
		// abbrechen und einen Fehler zurckliefern.
		//
		if( Data.length != 32 ) return null;
		if( !isValidCryptKey(CryptKey) ) return null;

		// Es werden acht Runden durchlaufen, nach jeder Runde werden die Teilschlssel
		// rotiert.
		//
		for( key_round=0; key_round < 8; key_round++ )
		{
			// Alle Teilrunden der Runde durchlaufen. Es werden alle Teilschlssel mit allen
			// Datenworten verknpft, dabei wird jedes Datenwort 8 mal verschlsselt.
			//
			for( key_subround=0; key_subround < 8; key_subround++ )
			{
				// In jeder Teilrunde jedes Datenworte mit je einem Teilschlssel verknpfen.
				//
				for( i=0; i < 32; i++ ) 
				{
					// Den Befehl aus dem Teilschlssel extrahieren und den Teilschlssel
					// selbst extrahieren (also ohne Befehl).
					//
					key = CryptKey[(key_subround*32)+i];
					key_cmd = (key >> 16) & 3;
					key = key & 0xffff;
		
					// Je nach Befehl die Verschlsselung durchfhren
					//
					switch( key_cmd )
					{
						// Im Falle einer NOT-Verschlsselung:
						//
						case 0:	Data[i] = (char)~Data[i];
								break;
	
						// Im Falle einer XOR-Verschlsselung:
						//
						case 1: Data[i] = (char)(((int)Data[i] ^ key) & 0xffff);
								break;
	
						// Im Falle einer ADD-Verschlsselung:
						//
						case 2: Data[i] = (char)(((Data[i] + key) % 0x10000) & 0xffff);
								break;
	
						// Im Falle einer SBOX-Verschlsselung:
						//
						case 3: // Zunchst ermitteln wir mit wem das Datenwort vertauscht werden soll
								key = key & 0x1f;
	
								// Wenn das Datenwort mit sich selbst vertauscht werden soll, dann
								// tun wir nichts (wozu auch ?)
								//
								if( key == i) break;
	
								// Sonst vertauschen wir die beiden Datenworte.
								//
								xchg = Data[key];
								Data[key] = Data[i];
								Data[i] = xchg;
								break;
					}
				}
			}

			// Zunchst werden alle Teilschlssel intern um 5 Bit nach links rotiert.
			//
			for( i=0; i < 256; i++ )
			{
				int overflow;

				key = CryptKey[i];
				overflow = (key >> 13) & 0x1f;
				key = key << 5;
				key = key & 0x3ffff;
				key = key + overflow;
				CryptKey[i] = key;
			}


			// Jetzt werden die Schlssel alle um 7 Schlssel nach rechts rotiert, dafr wird
			// zunchst eine Kopie des Originalschlssel angelegt, anschlieend wird die Kopie
			// zurck in den Originalschlssel kopiert, aber um 7 Schlssel nach rechts rotiert.
			//
			System.arraycopy( CryptKey, 0, key_cpy, 0, 256 );
			for( i=0; i < 256; i++ )
			{
				j = i + 7;
				if( j >= 256 ) j-=256;
			 	CryptKey[i] = key_cpy[j];
			}
		}
		return Data;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	<p>Diese Methode entschlsselt einen 64-Byte groen Datenblock.
	Der Methode mu ein 64-Byte groer verschlsselter Datenblock (siehe auch EncryptBlock) und 
	ein gltiger rotierter Crypt-Key bergeben werden. Der Datenblock mu aus 32 Worten 
	(16-Bit Werten) bestehen. Die Methode kann nur zum Entschlsseln verwendet werden. Die Methode 
	liefert im Erfolgsfall den sauberen (original) Datenblock zurck, sonst null. Null wird dann 
	zurckgeliefert, wenn die Methode einen ungltigen Datenblock (grer oder kleiner 32 Worten) 
	oder einen ungltigen Crypt-Key bergeben bekommt.</p>
	<p>
	<b>Wichtige Hinweise:</b><br>
	1.) Die Methode erwartet einen rotierten CryptKey wie er z.B. von der Methode "EncyptBlock" 
	zurckgeliefert wird. Diese Methode derotiert dann den entsprechenden Schlssel wieder. Um aus
	einen CryptKey einen rotierten CryptKey zu erzeugen kann die Methode "RotateCryptKey" verwendet
	werden.</p>
	<p>
	Der Vorteil dieses Verhaltens ist eine etwas hhere Performance in einige Situationen, der
	einfachste Weg immer mit einem Schlssel zu arbeiten ist einen EncryptKey und einen DecryptKey
	zu erzeugen und je nach bedarf nur eine Kopie des Originals an die Methoden EncryptBlock und 
	DecryptBlock zu bergeben.</p>
	<p>
	2.) Die Methode kann nicht berprfen ob der entschlsselte Block wirklich der Originalblock 
	ist oder der bergebene CryptKey der Richtige zum Entschlsseln war, denn es wird keine 
	Prfsumme in einem verschlsselten Block gespeichert anhand welcher die Methode dies berprfen 
	knnte. Fr solche Prfungen ist der Anwender selbst zustndig, auch dies wurde aus 
	Performancegrnden so gehalten, so mu nicht bei jedem Entschlsseln der Schlssel berprft
	werden was im Allgemeinen auch nicht ntig ist.</p>
	@param Data			Der Datenblock der emtschlsselt werden soll, dieser Block besteht aus 32
						Worten (16-Bit Werte).
	@param CryptKey		Der CryptKey mit welchem entschlsselt werden soll, nach dem erfolgreichen
						Entschlsseln ist dieser Schlssel derotiert.
	@return				Die Methode liefert 32 Worte (16-Bit Werte) zurck, wenn die 
						Entschlsselung erfolgreich durchgefhrt wurde, dabei handelt es sich um
						den bergebenen Datenblock, sonst null.
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	@see BlockCrypt#EncryptBlock
	@see BlockCrypt#RotateCryptKey
	@see BlockCrypt#DerotateCryptKey
	*/
	public char[] DecryptBlock( char[] Data, int[] CryptKey )
	{
		int i, j;								// Zhler
		char xchg = 0;							// Variabel zum vertauschen von zwei Datenworten
		int key;								// Schlssel und Reversschlssel fr ADD-Verschlsselung
		int key_cmd;							// Befehl aus einem Schlssel
		int key_subround;						// Teilrundenzhler
		int key_round;							// Rundenzhler
		int[] key_cpy = new int[256];			// Array zum Kopieren des Crypt-Key

		// Wenn der bergebene Datenblock oder der Crypt-Key ungltig ist, dann die Methode
		// abbrechen und einen Fehler zurckliefern.
		//
		if( Data.length != 32 ) return null;
		if( !isValidCryptKey(CryptKey) ) return null;

		// Es werden acht Runden durchlaufen, nach jeder Runde werden die Teilschlssel
		// rotiert.
		//
		for( key_round=0; key_round < 8; key_round++ )
		{
			// Jetzt werden die Schlssel alle um 7 Schlssel nach links rotiert, dafr wird
			// zunchst eine Kopie des Originalschlssel angelegt, anschlieend wird die Kopie
			// zurck in den Originalschlssel kopiert, aber um 7 Schlssel nach links rotiert.
			//
			System.arraycopy( CryptKey, 0, key_cpy, 0, 256 );
			for( i=0; i < 256; i++ )
			{
				j = i - 7;
				if( j < 0 ) j+=256;
			 	CryptKey[i] = key_cpy[j];
			}

			// Zunchst werden alle Teilschlssel intern um 5 Bit nach rechts rotiert.
			//
			for( i=0; i < 256; i++ )
			{
				int overflow;

				key = CryptKey[i];
				overflow = (key & 0x1f) << 13;
				key = key >> 5;
				key = key & 0x3ffff;
				key = key + overflow;
				CryptKey[i] = key;
			}

			// Alle Teilrunden der Runde durchlaufen. Es werden alle Teilschlssel mit allen
			// Datenworten verknpft, dabei wird jedes Datenwort 8 mal verschlsselt.
			//
			for( key_subround=7; key_subround >= 0; key_subround-- )
			{
				// In jeder Teilrunde jedes Datenworte mit je einem Teilschlssel verknpfen.
				//
				for( i=31; i >= 0; i-- ) 
				{
					// Den Befehl aus dem Teilschlssel extrahieren und den Teilschlssel
					// selbst extrahieren (also ohne Befehl).
					//
					key = CryptKey[(key_subround*32)+i];
					key_cmd = (key >> 16) & 3;
					key = key & 0xffff;
		
					// Je nach Befehl die Verschlsselung durchfhren
					//
					switch( key_cmd )
					{
						// Im Falle einer NOT-Verschlsselung:
						//
						case 0:	Data[i] = (char)~Data[i];
								break;
	
						// Im Falle einer XOR-Verschlsselung:
						//
						case 1: Data[i] = (char)(((int)Data[i] ^ key) & 0xffff);
								break;
	
						// Im Falle einer ADD-Verschlsselung:
						//
						case 2: Data[i] = (char)(((Data[i] - key) % 0x10000) & 0xffff);
								break;
	
						// Im Falle einer SBOX-Verschlsselung:
						//
						case 3: // Zunchst ermitteln wir mit wem das Datenwort vertauscht werden soll
								key = key & 0x1f;
	
								// Wenn das Datenwort mit sich selbst vertauscht werden soll, dann
								// tun wir nichts (wozu auch ?)
								//
								if( key == i) break;
	
								// Sonst vertauschen wir die beiden Datenworte.
								//
								xchg = Data[key];
								Data[key] = Data[i];
								Data[i] = xchg;
								break;
					}
				}
			}
		}
		return Data;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Methode rotiert einen Crypt-Key so das er zum Entschlsseln verwendet werden kann.
	Der Methode mu ein gtliger Crypt-Key bergeben werden. Die Methode rotiert diese Crypt-Key
	dann so, da er der Methode DecryptBlock bergeben werden kann. Diese Methode verndert den
	bergebenen Crypt-Key nicht, wenn im Parameter "copy" der Wert true bergeben wird, in
	diesem Fall wird eine rotierte Kopie des original Crypt-Keys zurckgegeben.
	@param CryptKey		Der CryptKey der rotiert werden soll.
	@param copy			Wenn hier true bergeben wird, dann wird der bergeben Crypt-Key nicht
						verndert, sondern es wird eine Kopie erzeugt und diese verndert.
	@return				Die Methode liefert den vernderten Crypt-Key zurck.
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	@see BlockCrypt#EncryptBlock
	@see BlockCrypt#RotateCryptKey
	@see BlockCrypt#DerotateCryptKey
	*/
	public int[] RotateCryptKey( int[] CryptKey, boolean copy )
	{
		int i, j;								// Zhler
		int key;								// Kopievariabel fr einen Schlssel
		int rotate_round;						// Rundenzhler zum Rotierern
		int[] key_cpy = new int[256];			// Array zum Kopieren des Crypt-Key
		int[] new_key;							// Array in welchem der Crypt-Key gendert wird

		// Wenn der bergebene Crypt-Key ungltig ist, dann die Methode abbrechen und einen 
		// Fehler zurckliefern.
		//
		if( !isValidCryptKey(CryptKey) ) return null;

		// Wenn der original CryptKey kopiert werden soll, also nicht verndert werden soll
		// dann dies jetzt tun, sonst einfach das Original rotieren
		//
		if( copy )
		{
			new_key = new int[256];
			System.arraycopy( CryptKey, 0, new_key, 0, 256 );
		} else
		{
			new_key = CryptKey;
		}

		for( rotate_round=0; rotate_round < 8; rotate_round++ )
		{
			// Zunchst werden alle Teilschlssel intern um 5 Bit nach links rotiert.
			//
			for( i=0; i < 256; i++ )
			{
				int overflow;

				key = new_key[i];
				overflow = (key >> 13) & 0x1f;
				key = key << 5;
				key = key & 0x3ffff;
				key = key + overflow;
				new_key[i] = key;
			}


			// Jetzt werden die Schlssel alle um 7 Schlssel nach rechts rotiert, dafr wird
			// zunchst eine Kopie des Originalschlssel angelegt, anschlieend wird die Kopie
			// zurck in den Originalschlssel kopiert, aber um 7 Schlssel nach rechts rotiert.
			//
			System.arraycopy( new_key, 0, key_cpy, 0, 256 );
			for( i=0; i < 256; i++ )
			{
				j = i + 7;
				if( j >= 256 ) j-=256;
			 	new_key[i] = key_cpy[j];
			}
		}
		return new_key;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	Diese Methode gewinnt aus einen rotierten Crypt-Key wieder das Original.
	Der Methode mu ein gtliger Crypt-Key bergeben werden welcher rotiert ist. Die Methode 
	rotiert diese Crypt-Key dann so in die andere Richtung, da er der Methode CryptBlock wieder
	bergeben werden kann. Diese Methode verndert den bergebenen Crypt-Key nur wenn im Parameter
	"copy" der Wert false bergeben wird, sonst wird der bergebene Crypt-Key kopiert und die
	Kopie rotiert.
	@param CryptKey		Der CryptKey der rotiert werden soll.
	@param copy			Wenn hier true bergeben wird, dann wird der bergeben Crypt-Key nicht
						verndert, sondern es wird eine Kopie erzeugt und diese verndert.
	@return				Die Methode liefert den vernderten Crypt-Key zurck.
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	@see BlockCrypt#EncryptBlock
	@see BlockCrypt#RotateCryptKey
	@see BlockCrypt#DerotateCryptKey
	*/
	public int[] DerotateCryptKey( int[] CryptKey, boolean copy )
	{
		int i, j;								// Zhler
		int key;								// Kopievariabel fr einen Schlssel
		int rotate_round;						// Rundenzhler zum Rotierern
		int[] key_cpy = new int[256];			// Array zum Kopieren des Crypt-Key
		int[] new_key;							// Array in welchem der Crypt-Key gendert wird

		// Wenn der bergebene Crypt-Key ungltig ist, dann die Methode abbrechen und einen 
		// Fehler zurckliefern.
		//
		if( !isValidCryptKey(CryptKey) ) return null;

		// Wenn der original CryptKey kopiert werden soll, also nicht verndert werden soll
		// dann dies jetzt tun, sonst einfach das Original rotieren
		//
		if( copy )
		{
			new_key = new int[256];
			System.arraycopy( CryptKey, 0, new_key, 0, 256 );
		} else
		{
			new_key = CryptKey;
		}

		for( rotate_round=0; rotate_round < 8; rotate_round++ )
		{
			// Jetzt werden die Schlssel alle um 7 Schlssel nach links rotiert, dafr wird
			// zunchst eine Kopie des Originalschlssel angelegt, anschlieend wird die Kopie
			// zurck in den Originalschlssel kopiert, aber um 7 Schlssel nach links rotiert.
			//
			System.arraycopy( new_key, 0, key_cpy, 0, 256 );
			for( i=0; i < 256; i++ )
			{
				j = i - 7;
				if( j < 0 ) j+=256;
			 	new_key[i] = key_cpy[j];
			}

			// Zunchst werden alle Teilschlssel intern um 5 Bit nach rechts rotiert.
			//
			for( i=0; i < 256; i++ )
			{
				int overflow;

				key = new_key[i];
				overflow = (key & 0x1f) << 13;
				key = key >> 5;
				key = key & 0x3ffff;
				key = key + overflow;
				new_key[i] = key;
			}
		}
		return new_key;
	}
//=================================================================================================



//-------------------------------------------------------------------------------------------------
	/**	<p>Diese Methode berprft ein Kennwort (als StringKey) auf seine Sicherheit.
	Der Methode wird ein Kennwort als StringKey bergeben und die Methode berprft dieses Kennwort 
	auf sein Sicherheit, wenn das Kennwort als "unsicher" anzusehen ist, dann liefert die Methode 
	false zurck und das entsprechende Kennwort sollte von einer Verschlsselungssoftware auf jeden
	Fall abgelehnt werden. Besonder wichtig ist diese Methode fr zufllig erzeugte Kennwrter,
	welche z.B. bei als temporre Kennwrter benutzt werden sollen.</p>
	<p>
	Als unsicher gelten alle Kennwrter die aus Bitkombinationen bestehen die eine schlechte
	Bitverteilung aufweisen, also z.B. zu mehr 90% aus gesetzen oder gelschten Bits bestehen. 
	Weiterhin sind alle Kennwrter unsicher, die nicht zulig sind, also weniger als 12 Zeichen 
	enthalten. Zustzlich sind zu einfache Kennwrter unsicher, also Kennwrter die zu mehr als
	90% aus Kleinbuchstaben, oder zu mehr als 70% aus Grobuchstaben bzw. zu mehr als 70% aus
	Ziffern bestehen. Sonderzeichen drfen nicht mehr als 25% enthalten sein. Kennwrter
	die lnger als 47 Zeichen sind und eine gute Bitverteilung aufweisen sind in jeden Fall
	sicher.</p>
	@param sPassword	Das Kennwort welches auf seine Sicherheit berprft werden soll.
	@return				Die Methode liefert true zurck, wenn das bergebene Kennwort nach 
						heutiger sicht als "sicher" angesehen werden kann.
	@see <a href="vsc.html">Dokumentation (Very Simple Crypt)</a>
	@see BlockCrypt#EncryptBlock
	@see BlockCrypt#RotateCryptKey
	@see BlockCrypt#DerotateCryptKey
	*/
	public boolean isSafeCryptString( int[] StringKey )
	{
		int i, j;								// Zhler
		int set_cnt = 0, unset_cnt = 0;			// Zhler fr die gesetzten und ungesetzt Bits
		int[] char_cnt = new int[64];			// Zhler fr alle Zeichen.
		int char_A2Z = 0;						// Zhler fr die verschiedenen Arten von Zeichen
		int char_a2z = 0;
		int char_029 = 0;
		int char_rest = 0;
		int chars = 0;							// Anzahl der gltigen Zeichen insgesamt
		int tmp;								// Temporre Variabel fr Zwischenoperationen

		// Wenn kein Kennwort bergeben wurde, kann es auch nicht sicher sein.
		//
		if( StringKey == null ) return false;

		// Zunchst initialisieren wird die Zeichenzhler.
		//
		for( i=0; i < 64; i++ ) char_cnt[i] = 0;

		// Jetzt zhlen wir wieviele Bits in dem Zeichenschlssel gesetzt und wie viele ungesetzt
		// sind, dabei gilt natrlich daraus zu achten nur aus gltigen Zeichenschlsseln zu
		// lesen. Auerdem zhlen wir welche Zeichen wie oft in der Zeichenkette vorkommen.
		//
		for( i=0; i < StringKey.length; i++ ) {
			// Wenn das Zeichen gltig ist
			//
			if(  (StringKey[i] >= 0) && (StringKey[i] <= 63)  )
			{
				// Zeichenzhler erhhen und den Zhler fr das entsprechende Zeichen erhhen.
				// Auerdem noch den Zhler fr Kategorie des Zeichens erhhen.
				//
				chars++;
				char_cnt[ StringKey[i] ]++;
				if(  (StringKey[i] >= 00) && (StringKey[i] <= 25)  ) char_A2Z++;
				if(  (StringKey[i] >= 26) && (StringKey[i] <= 51)  ) char_a2z++;
				if(  (StringKey[i] >= 52) && (StringKey[i] <= 61)  ) char_029++;
				if(  (StringKey[i] == 62) || (StringKey[i] == 63)  ) char_rest++;

				// Jetzt zhlen wir die Anzahl der gesetzten bzw. ungesetzten Bits
				//
				tmp = StringKey[i];
				for( j=0; j < 6; j++)
				{
					if( (tmp & 1) == 1 ) set_cnt++;
									else unset_cnt++;
					tmp = tmp >> 1;
				}
			}
		}

		// Wenn weniger als 12 Zeichen gltig sind, dann ist das Kennwort unsicher
		//
		if( chars < 12 ) return false;

		// Wenn mehr als 90% der Bits gesetzt oder ungesetzt sind, dann ist das Kennwort unsicher.
		//
		if( ((set_cnt*100) / (chars*6)) > 90 ) return false;
		if( ((unset_cnt*100) / (chars*6)) > 90 ) return false;

		// Wenn das Kennwort lnger als 47 Zeichen ist, dann brauchen wir uns wohl keine gedanken
		// mehr um die Verteilung der Buchstaben zu machen. Das Kennwort ist dann sicher.
		//
		if( chars > 47 ) return true;

		// Wenn die Zeichenkette zu mehr als 70% aus Groschrift oder Ziffern besteht, zu mehr
		// als 90% aus Kleinschrift oder zu mehr als 25% aus Sonderzeichen, dann gilt das Kennwort 
		// als unsicher.
		//
		if( ((char_A2Z*100) / chars) > 70 ) return false;
		if( ((char_a2z*100) / chars) > 90 ) return false;
		if( ((char_029*100) / chars) > 70 ) return false;
		if( ((char_rest*100) / chars) > 30 ) return false;

		// Wenn eines der Zeichen zu mehr als 25% enthalten ist, dann ist das Kennwort unsicher
		//
		for( i=0; i < 64; i++ )
			if( ((char_cnt[i]*100) / chars) > 25 ) return false;

		// Sonst ist das Kennwort nach derzeitigem Stand sicher
		//
		return true;
	}
//=================================================================================================


} /* Ende der Klasse */
