import java.io.*;
import BlockCrypt;

/**	<p>Diese Klasse kann Dateien ver- und wieder entschlsseln.
Sie benutzt dazu die Klasse BlockCrypt, welche die Crypt-Schnittstelle implementiert. Die
Crypt-Schnittstelle erlaubt das Ver- und Entschlsseln von 64 Byte groen Datenblcken nach einem
sehr sicheren und schnellen Algorithmus. Der Algorithmus wird in der folgenden
<a href="vsc.html">Dokumentation</a> genauer erleutert, Sie knnen auch den Quellcode der
Implementation (VSC) einsehen.</p>
<p>
Wenn Sie eine grafische Oberflche oder ein Applet schreiben mchten welches Dateien verschlsselt,
dann leiten Sie einfach ihre neue Klasse von dieser ab oder erzeugen Sie ein Objekt der Klasse
FileCrypt, Sie knnen dann die Methoden EncryptFile bzw. DecryptFile zum ver- bzw. entschlsseln
von Dateien beliebiger Gre benutzen.</p>
<p>
Wenn Sie die Klasse als Programm von einem Kommandozeileninterpreter aufrufen mchten, dann
geben Sie ein: java FileCrypt <e|d> <Dateiname> <Kennwort></p>
<p>
Mit dem Parameter <e|d> ist entweder das Zeichen 'e' fr encrypt (Verschlsseln) oder 'd' fr
decrypt (Entschlsseln) gemeint. Sie knnen eine Datei mehrfach verschlsseln, bitte beachten Sie
aber das Sie diese Datei dann auch wieder entsprechend oft entschlsseln mssen.</p>
<p>
Der Parameter <Dateiname> gibt entweder den Dateinamen der zu verschlsselnden Originaldatei, oder
den Dateinamen der zu entschlsselnden Datei. Dabei wird der Dateiname immer mit Pfad und
Dateierweiterung angegeben. Beim verschlsseln wird eine Zieldatei erzeugt die im gleichen Pfad
wie die Originaldatei liegt und die Dateierweiterung .VSC trgt, diese Erweiterung wird einfach
an den Dateinamen angehngt. Beim entschlsseln wird der Name der Quelldatei aus dem Dateinamen
plus der Erweiterung .VSC gebildet, die Zieldatei wird dort erzeugt wo die Quelldatei liegt und
ihr wird die Erweiterung .VSC wieder entnommen. Beispiel:</p>
<p>
Die Anweisung "java FileCrypt e test.txt Dies ist mein Kennwort" erzeugt eine Datei mit dem
Namen "text.txt.vsc". Die Anweisung "java FileCrypt d test.txt Dies ist mein Kennwort" restauriert
dann wieder die Originaldatei "test.txt" aus der Datei "test.txt.vsc".</p>
<p>
Das Kennwort darf nur aus den Zeichen 'a' bis 'z', 'A' bis 'Z', '0' bis '9', dem Punkt und
Leerzeichen bestehen. Es mu mindestens 12 Zeichen lang sein und darf hchstens 768 Zeichen
enthalten, wobei ich mir nicht Vorstellen kann wer ein so langes Kennwort angibt, auerdem bin
ich mir nicht sicher ob MS-DOS berhaupt Parameter mit solcher Lnge bergeben kann, denke aber
eher nicht. Empfehlen wrde ich ein Kennwort mit ca. 48 Zeichen, das ist absolut lang genug und
auch definitiv Sicher. Um ein Kennwort der Mindestlnge definitiv Sicher zu erraten mte ein
Angreifer denn auch 64<sup>12</sup> Versuche wagen, denn ein solches Kennwort enthlt fr jedes
Zeichen 64 Mglichkeiten was zu insgesamt 4,722,366,482,869,645,213,696 mglichen Kombinationen
fhrt. Damit drfte man bereits weit auerhalb der heutigen Rechenleistung liegen, selbst wenn
ein Computer 1,000,000,000 (eine Milliarde) Mglichkeiten pro Sekunde ausprobieren knnte,
wrde er mit einem Brute-Force Angriff bei 12 Zeichen im ungnstigsten Fall 4,722,366,482,869
Sekunden oder 1,311,768,467 Stunden oder 149,745 Jahre brauchen um das entsprechende Kennwort
zu brechen.</p>
<p>
<hr>
    This program is free software; you can redistribute it and/or modify							<br>
    it under the terms of the GNU General Public License as published by							<br>
    the Free Software Foundation; either version 2 of the License, or								<br>
    (at your option) any later version.																<br>
																									<br>
    This program is distributed in the hope that it will be useful,									<br>
    but WITHOUT ANY WARRANTY; without even the implied warranty of									<br>
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the									<br>
    GNU General Public License for more details.													<br>
																									<br>
    You should have received a copy of the GNU General Public License								<br>
    along with this program; if not, write to the Free Software										<br>
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA						<br>
<hr>
</p>



<p>
<b>Bekannte Fehler:</b></p>
<p><table border=1>

<tr><th align=left>
keine Fehler bekannt.
</th></tr>

</table></p>



<p>
<b>nderungen an der Klasse:</b></p>
<p><table border=1>

<tr><th align=left width=20%>29.11.2000<br>Alexander Weber<br>--[Build 1]--</th>
<th align=left>
	Die Klasse wird von mir zum ersten mal released.
</th></tr>

</table></p>

@author (C) 2000, <a href="mailto:Alexander.Weber@myokay.net">Alexander Weber</a>
@version Version 1.0alpha (Build 1)
*/
public class FileCrypt
extends BlockCrypt
{

//-------------------------------------------------------------------------------------------------
	/** Kein Fehler */
	static final int FILECRYPT_OK				= 0;
	/** Datei nicht gefunden (ffnen der Quelldatei) */
	static final int FILECRYPT_ERR_FILENOTFOUND	= 1;
	/** Datei kann nicht geffnet bzw. angelegt werden (erzeugen der Zieldatei) */
	static final int FILECRYPT_ERR_FILECREATE	= 2;
	/** Fehler beim Lesen aus der Datei (Quelldatei) */
	static final int FILECRYPT_ERR_FILEREAD		= 3;
	/** Fehler beim Schreiben in die Datei (Zieldatei) */
	static final int FILECRYPT_ERR_FILEWRITE	= 4;
	/** Fehler beim Schlieen der Datei (Quell- oder Zieldatei) */
	static final int FILECRYPT_ERR_CLOSE		= 5;
	/** Fehler im Schlssel, wahrscheinlich ein zu kurzes Kennwort angegeben */
	static final int FILECRYPT_ERR_KEY			= 6;
	/** Die Quelldatei ist nicht oder mit einem anderen Kennwort verschlsselt */
	static final int FILECRYPT_ERR_NOTENCRYPTED	= 7;

//-------------------------------------------------------------------------------------------------



//-------------------------------------------------------------------------------------------------
	/** Diese Methode verschlsselt eine Datei.
	Die bergebene Quelldatei wird verschlsselt und als Zieldatei abgespeichert, dabei gilt zu 
	beachten das die Zieldatei mindestens 64 Byte grer wird als die Quelldatei ist. Auerdem 
	wird die Zieldatei berschrieben, falls Sie bereits existiert. Die Zieldatei erhlt die
	Dateilnge der Quelledatei + 64 Byte und aufgerundet auf das nchste 64 Byte Aligment.

	@param source		Dateiname der Quelldatei, diese Datei wird verschlsselt.
	@param dest			Dateiname der Zieldatei, diese Datei wird berschrieben bzw. falls Sie
						noch nicht existiert erzeugt.
	@param password		Hier wird das Kennwort bergeben mit welchem die Datei verschlsselt
						werden soll. Das Kennwort darf nur aus folgenden Zeichen/Ziffern 
						bestehen: 'A' bis 'Z', 'a' bis 'z', '0' bis '9' dem Punkt ('.') und
						aus Leerzeichen. Alle anderen Zeichen werden ignoriert, auerdem gilt
						zu beachten das mindestens 12 gltige Zeichen enthalten sein men und
						das nur Anzahl/3 Zeichen signifikant sind, also das Kennwort 
						"123456789012" ist gleich dem Kennwort "1234567890123" !
	@param OutputErr	Wenn hier false bergeben wird, dann unterdrckt die Methode smmtliche
						Fehlermeldungen, es erscheinen keine Fehlermeldungen auf dem 
						Standardausgabegert. Bei true gibt die Funktion im Fehlerfall eine
						Fehlermeldung auf dem Standardausgabegert aus.
	@return 			Die Methode liefert einen Fehlercode zurck der einer der weiter oben
						angegebenen Konstanten entspricht.
	*/
	public int EncryptFile( String source, String dest, String password, boolean OutputErr )
	{
		int[] EncryptKey;						// Der original Verschlsselungsschlssel
		int[] CryptKey = new int[256];			// Eine Kopie des original Schlssels
		RandomAccessFile source_file;			// Quelldatei
		RandomAccessFile dest_file;				// Zieldatei
		String FileID = new String("VSC Encrypted File");
		long filesize;							// Dateigre
		long i, j;								// Zhler
		long blocks;							// Anzahl der zu verschlsselnden Blocks
		long lastblocksize;						// Gre des letzten zu verschlsselnden Blocks
		char[] data = new char[32];				// Datenblock der Verschlsselt wird

		// Den Verschlsselungsschlssel (Encrypt-Key) aus dem Kennwort generieren.
		//
		EncryptKey = QueryCompleteCryptKey( password );

		// Wenn der Schlssel ungltig ist mit einer Fehlermeldung abbrechen
		//
		if( !isValidCryptKey( EncryptKey ) )
		{
			if( OutputErr ) System.out.println( "Fehler beim Schluessel, wahrscheinlich weniger als 12 Zeichen !" );
			return FILECRYPT_ERR_KEY;
		}

		// Die Quelldatei ffnen.
		//
		try	{
			source_file = new RandomAccessFile( source, "r" );
		} catch(FileNotFoundException e) {
			if( OutputErr ) System.out.println( "Fehler beim oeffnen der Quelldatei !" );
			return FILECRYPT_ERR_FILENOTFOUND;
		}

		// Die Zieldatei ffnen bzw. erzeugen
		//
		try {
			dest_file = new RandomAccessFile( dest, "rw" );
		} catch(FileNotFoundException e) {
			if( OutputErr ) System.out.println( "Fehler beim oeffnen/erzeugen der Zieldatei !" );
			return FILECRYPT_ERR_FILECREATE;
		}

		// Die Dateigre der Quelldatei ermitteln
		//
		try	{	
			filesize = source_file.length();
		} catch(IOException e) {
			if( OutputErr ) System.out.println( "Fehler beim Lesen aus der Quelldatei !" );
			return FILECRYPT_ERR_FILEREAD;
		}

		// Anzahl der zu verschlsselnden Blocks ermitteln und die Gre des letzten Blocks
		// berechnen, der jener welcher brig bleibt.
		//
		blocks = filesize / 64;
		lastblocksize = filesize - (blocks * 64);

		// Zunchst die Identifikation und die Dateilnge der Originaldatei in die Zieldatei 
		// schreiben (beides verschlsselt !).
		//
		for( i=0; i < 32; i++ ) data[(int)i] = 0;
		for( i=0; i < 18; i++ ) data[(int)i] = FileID.charAt((int)i);
		data[18] = (char)(filesize % 0x10000);
		data[19] = (char)((filesize >> 16) % 0x10000);
		data[20] = (char)((filesize >> 32) % 0x10000);
		data[21] = (char)((filesize >> 48) % 0x10000);

		// Eine Kopie des Originalschlssels anlegen
		//
		System.arraycopy( EncryptKey, 0, CryptKey, 0, 256 );

		// Datenblock verschlsseln und in die Zieldatei schreiben.
		//
		try {
			EncryptBlock( data, CryptKey );
			for( j=0; j < 32; j++ ) dest_file.writeChar( data[(int)j] );
		} catch(IOException e) {
			if( OutputErr ) System.out.println( "Fehler beim Schreiben in die Zieldatei !" );
			return FILECRYPT_ERR_FILEWRITE;
		}

		// Zunchst verschlsseln wir alle vollstndigen Blcke
		//
		for( i=0; i < blocks; i++ )
		{
			// Datenblock aus der Quelldatei lesen, verschlsseln und in die Zieldatei 
			// schreiben.
			//
			try {
				for( j=0; j < 32; j++ ) data[(int)j] = source_file.readChar();
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Lesen aus der Quelldatei !" );
				return FILECRYPT_ERR_FILEWRITE;
			}
			EncryptBlock( data, CryptKey );
			try {
				for( j=0; j < 32; j++ ) dest_file.writeChar( data[(int)j] );
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Schreiben in die Zieldatei !" );
				return FILECRYPT_ERR_FILEWRITE;
			}
		}

		// Jetzt werden wir auch den letzten Block, falls er denn existiert, noch verschlsseln und 
		// in die Datei schreiben.
		//
		if( lastblocksize > 0 )
		{
			// Den Datenblock mit Nullen fllen
			//
			for( j=0; j < 32; j++ ) data[(int)j] = 0;

			// Datenblock aus der Quelldatei lesen, verschlsseln und in die Zieldatei 
			// schreiben.
			//
			try {
				for( j=0; j < (lastblocksize/2); j++ ) data[(int)j] = source_file.readChar();
				if( (lastblocksize % 2) == 1 ) data[(int)j] = (char)source_file.readUnsignedByte();
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Lesen aus der Quelldatei !" );
				return FILECRYPT_ERR_FILEREAD;
			}
			EncryptBlock( data, CryptKey );
			try {
				for( j=0; j < 32; j++ ) dest_file.writeChar( data[(int)j] );
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Schreiben in die Zieldatei !" );
				return FILECRYPT_ERR_FILEWRITE;
			}
		}

		// Die noch offene Quell- und Zieldatei schlieen
		//
		try {
			source_file.close();
			dest_file.close();
		} catch(IOException e) {
			if( OutputErr ) System.out.println( "Fehler beim Schliessen der Quell- oder Zieldatei !" );
			return FILECRYPT_ERR_FILEWRITE;
		}

		// Wenn alles geklappt hat, dann toll
		//
		return FILECRYPT_OK;
	}
//-------------------------------------------------------------------------------------------------



//-------------------------------------------------------------------------------------------------
	/** Diese Methode entschlsselt eine Datei.
	Die bergebene Quelldatei wird entschlsselt und als Zieldatei abgespeichert, dabei gilt zu 
	beachten das die Zieldatei berschrieben wird, falls Sie bereits existiert.

	@param source		Dateiname der Quelldatei, diese Datei mu mit dem bergebenen Kennwort
						verschlsselt worden sein.
	@param dest			Dateiname der Zieldatei, diese Datei wird berschrieben bzw. falls Sie
						noch nicht existiert erzeugt.
	@param password		Hier wird das Kennwort bergeben mit welchem die Datei verschlsselt
						wurde. Das Kennwort darf nur aus folgenden Zeichen/Ziffern bestehen: 
						'A' bis 'Z', 'a' bis 'z', '0' bis '9' dem Punkt ('.') und aus
						Leerzeichen. Alle anderen Zeichen werden ignoriert, auerdem gilt zu
						beachten das mindestens 12 gltige Zeichen enthalten sein men und das
						nur Anzahl/3 Zeichen signifikant sind, also das Kennwort "123456789012" 
						ist gleich dem Kennwort "1234567890123" !
	@param OutputErr	Wenn hier false bergeben wird, dann unterdrckt die Methode smmtliche
						Fehlermeldungen, es erscheinen keine Fehlermeldungen auf dem 
						Standardausgabegert. Bei true gibt die Funktion im Fehlerfall eine
						Fehlermeldung auf dem Standardausgabegert aus.
	@return 			Die Methode liefert einen Fehlercode zurck der einer der weiter oben
						angegebenen Konstanten entspricht.
	*/
	public int DecryptFile( String source, String dest, String password, boolean OutputErr )
	{
		int[] DecryptKey;						// Der original Entschlsselungsschlssel
		int[] CryptKey = new int[256];			// Eine Kopie des original Schlssels
		RandomAccessFile source_file;			// Quelldatei
		RandomAccessFile dest_file;				// Zieldatei
		String FileID;
		long filesize;							// Dateigre
		long i, j;								// Zhler
		long blocks;							// Anzahl der zu verschlsselnden Blocks
		long lastblocksize;						// Gre des letzten zu verschlsselnden Blocks
		char[] data = new char[32];				// Datenblock der Verschlsselt wird

		// Jetzt erzeugen wir den Decrypt-Key aus dem Kennwort.
		//
		DecryptKey = QueryCompleteCryptKey( password );

		// Wenn der Schlssel ungltig ist mit einer Fehlermeldung abbrechen
		//
		if( !isValidCryptKey( DecryptKey ) )
		{
			if( OutputErr ) System.out.println( "Fehler beim Schluessel, wahrscheinlich weniger als 12 Zeichen !" );
			return FILECRYPT_ERR_KEY;
		}
		DecryptKey = RotateCryptKey( DecryptKey, false);

		// Die Quelldatei ffnen und die Dateigre der Originaldatei auslesen.
		//
		try {
			source_file = new RandomAccessFile( source, "r" );
		} catch(FileNotFoundException e) {
			if( OutputErr ) System.out.println( "Fehler beim oeffnen der Quelldatei !" );
			return FILECRYPT_ERR_FILENOTFOUND;
		}

		// Zunchst die Identifikation und die Dateilnge der Originaldatei aus der Quelldatei
		// lesen (nu entschlsselt werden !).
		//
		try {
			for( i=0; i < 32; i++ ) data[(int)i] = source_file.readChar();
		} catch(IOException e) {
			if( OutputErr ) System.out.println( "Fehler beim Lesen aus der Quelldatei !" );
			return FILECRYPT_ERR_FILEREAD;
		}
		
		// Eine Kopie des Originalschlssels anlegen
		//
		System.arraycopy( DecryptKey, 0, CryptKey, 0, 256 );

		// Datenblock entschlsseln und die Dateiidentifikation sowie die Dateilnge extrahieren.
		//
		DecryptBlock( data, CryptKey ); RotateCryptKey( DecryptKey, false);
		FileID = new String( data, 0, 18 );
		filesize = (long)data[18] + (((long)data[19]) << 16) + (((long)data[20]) << 32) + (((long)data[21]) << 48);

		// berprfen wir ob es sich berhaupt um eine verschlsselte Datei handelt.
		//
		if( !FileID.equals("VSC Encrypted File") )
		{
			if( OutputErr ) 
			{
				System.out.println( "Die Quelldatei ("+source+") ist eine ungueltige Datei !" );
				System.out.println( "Entweder die Datei wurde nicht mit diesem Programm verschluesselt, oder " );
				System.out.println( "es wurde ein anderes Kennwort als das von ihnen angegebene verwendet." );
			}
			return FILECRYPT_ERR_NOTENCRYPTED;
		}

		// Jetzt die Zieldatei ffnen/erzeugen.
		//
		try {
			dest_file = new RandomAccessFile( dest, "rw" );
		} catch(FileNotFoundException e) {
			if( OutputErr ) System.out.println( "Fehler beim oeffnen/erzeugen der Zieldatei !" );
			return FILECRYPT_ERR_FILENOTFOUND;
		}

		// Anzahl der verschlsselten Blocks ermitteln und die Gre des letzten Blocks
		// berechnen, der jener welcher brig bleibt.
		//
		blocks = filesize / 64;
		lastblocksize = filesize - (blocks * 64);

		// Zunchst entschlsseln wir alle vollstndigen Blcke
		//
		for( i=0; i < blocks; i++ )
		{
			// Eine Kopie des Originalschlssels anlegen
			//
			System.arraycopy( DecryptKey, 0, CryptKey, 0, 256 );

			// Datenblock aus der Quelldatei lesen, entschlsseln und in die Zieldatei 
			// schreiben.
			//
			try {
				for( j=0; j < 32; j++ ) data[(int)j] = source_file.readChar();
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Lesen aus der Quelldatei !" );
				return FILECRYPT_ERR_FILEREAD;
			}
			DecryptBlock( data, CryptKey ); RotateCryptKey( DecryptKey, false);
			try {
				for( j=0; j < 32; j++ ) dest_file.writeChar( data[(int)j] );
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Schreiben in die Zieldatei !" );
				return FILECRYPT_ERR_FILEWRITE;
			}
		}

		// Jetzt werden wir auch den letzten Block, falls er denn existiert,
		// noch entschlsseln und in die Datei schreiben.
		//
		if( lastblocksize > 0 )
		{
			// Den Datenblock mit Nullen fllen
			//
			for( j=0; j < 32; j++ ) data[(int)j] = 0;

			// Eine Kopie des originalschlssels anlegen
			//
			System.arraycopy( DecryptKey, 0, CryptKey, 0, 256 );

			// Datenblock aus der Quelldatei lesen, entschlsseln und in die Zieldatei 
			// schreiben.
			//
			try {
				for( j=0; j < 32; j++ ) data[(int)j] = source_file.readChar();
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Lesen aus der Quelldatei !" );
				return FILECRYPT_ERR_FILEREAD;
			}
			DecryptBlock( data, CryptKey ); RotateCryptKey( DecryptKey, false);
			try
			{
				for( j=0; j < (lastblocksize/2); j++ ) dest_file.writeChar( data[(int)j] );
				if( (lastblocksize % 2) == 1 ) 
				{
					data[(int)j] = (char)(data[(int)j] & 0xff);
					dest_file.writeByte( (byte)data[(int)j] );
				}
			} catch(IOException e) {
				if( OutputErr ) System.out.println( "Fehler beim Schreiben in die Zieldatei !" );
				return FILECRYPT_ERR_FILEWRITE;
			}
		}

		// Die noch offene Quell- und Zieldatei schlieen
		//
		try {
			source_file.close();
			dest_file.close();
		} catch(IOException e) {
			if( OutputErr ) System.out.println( "Fehler beim Schliessen der Quell- oder Zieldatei !" );
			return FILECRYPT_ERR_FILEWRITE;
		}

		// Wenn alles geklappt hat, dann toll
		//
		return FILECRYPT_OK;
	}
//-------------------------------------------------------------------------------------------------



//-------------------------------------------------------------------------------------------------
	/** Programmeintritt, falls dieses Klasse als Anwendung gestartet wird.
	*/
	public static void main( String[] args )
	{
		FileCrypt self = new FileCrypt();
		int i = 0;								// Zhler

		System.out.println( "FileCrypt version 1.0alpha (build 1)" );
		System.out.println( "Copyright (C)2000 by Alexander Weber" );
		System.out.println( "This software comes with ABSOLUTELY NO WARRANTY" );
		System.out.println( "This is free software, and you are welcome to redistribute it" );
		System.out.println( "under certain conditions; you can find the GPL as textfile in" );
		System.out.println( "the same directory (gpl.txt)" );
		System.out.println( "(GPL = Gerneral Public Liecens)" );
		System.out.println();
		System.out.println( "Special Warning: This is no realease, it is alpha code !" );
		System.out.println();

		// Wenn nicht mehr als drei Parameter bergeben wurde, dann kann nicht gestartet werden.
		// 
		if( args.length < 3 )
		{
			System.out.println( "Syntax: java FileCrypt <e|d> <Dateiname> <Kennwort>" );
			System.out.println( "e = Encrypt (Verschluesseln)" );
			System.out.println( "d = Decrypt (Entschluesseln)" );
		}
		else
		{
			// Zunchst den Dateinamen extrahieren
			//
			String filename = new String( args[1] );
			String password = new String( args[2] );

			// Wenn noch mehr als die 3 Parameter bergeben wurden, dann betrachten wird diese
			// Zusatzparameter als Teile des Kennwort.
			//
			for( i=3; i < args.length; i++ ) password += args[i];

			// Wenn eine Datei verschlsselt werden soll, dann die entsprechende Methode aufrufen
			//
			if( args[0].charAt(0) == 'e' ) 
			{
				int result;
				// Jetzt wird das Kennwort auf seine Sicherheit berprft, wenn das Kennwort
				// als unsicher gilt, dann wird die Verschlsselung damit abgelehnt, sonst
				// kann's los gehen
				//
				if( !self.isSafeCryptString(self.QueryCryptString(password)) )
				{
					System.out.println( "Das von ihnen angegebene Kennwort wurde als 'unsicher' ausgewiesen !" );
					System.out.println( "Bitte verwenden Sie ein anderes sichereres Kennwort in ihrem" );
					System.out.println( "eigenen Interesse, Danke." );
					return;
				}
				System.out.println( "Try encrypting of "+filename+" to "+filename+".vsc" );
				System.out.println( "Using Password :"+password );
				result = self.EncryptFile( filename, filename+".vsc", password, true );
				if( result == FILECRYPT_OK ) System.out.println( "File successfull encrypted !" );
				return;
			}

			// Wenn eine Datei entschlsselt werden soll, dann die entsprechende Methode aufrufen
			//
			if( args[0].charAt(0) == 'd' )
			{
				int result;

				System.out.println( "Try decrypt of "+filename+".vsc to "+filename );
				System.out.println( "Using Password :"+password );
				result = self.DecryptFile( filename+".vsc", filename, password, true );
				if( result == FILECRYPT_OK ) System.out.println( "File successfull decrypted !" );
				return;
			}

			// Wenn ein ungltiger Parameter bergeben wurde, dann eine Fehlermeldung ausgeben
			//
			System.out.println( "Syntax: java FileCrypt <e|d> <Dateiname> <Kennwort>" );
			System.out.println( "e = Encrypt (Verschluesseln)" );
			System.out.println( "d = Decrypt (Entschluesseln)" );
		}
	}
//-------------------------------------------------------------------------------------------------
}


